@@ -2,12 +2,14 @@ import logging
 import re
 from collections.abc import Sequence
 
-from mitmproxy import ctx, exceptions
-from mitmproxy.addons.modifyheaders import parse_modify_spec, ModifySpec
+from mitmproxy import ctx
+from mitmproxy import exceptions
+from mitmproxy.addons.modifyheaders import ModifySpec
+from mitmproxy.addons.modifyheaders import parse_modify_spec
 
 
 class ModifyBody:
-    def __init__(self):
+    def __init__(self) -> None:
         self.replacements: list[ModifySpec] = []
 
     def load(self, loader):
