@@ -1,15 +1,26 @@
 import ipaddress
 import logging
 import os
+import ssl
 from pathlib import Path
-from typing import Any, Optional, TypedDict
+from typing import Any
+from typing import TypedDict
 
-from OpenSSL import SSL, crypto
-from mitmproxy import certs, ctx, exceptions, connection, tls
+from aioquic.h3.connection import H3_ALPN
+from aioquic.tls import CipherSuite
+from OpenSSL import crypto
+from OpenSSL import SSL
+
+from mitmproxy import certs
+from mitmproxy import connection
+from mitmproxy import ctx
+from mitmproxy import exceptions
+from mitmproxy import tls
 from mitmproxy.net import tls as net_tls
 from mitmproxy.options import CONF_BASENAME
 from mitmproxy.proxy import context
 from mitmproxy.proxy.layers import modes
+from mitmproxy.proxy.layers import quic
 from mitmproxy.proxy.layers import tls as proxy_tls
 
 # We manually need to specify this, otherwise OpenSSL may select a non-HTTP2 cipher by default.
@@ -52,8 +63,8 @@ DEFAULT_HOSTFLAGS = (
 
 
 class AppData(TypedDict):
-    client_alpn: Optional[bytes]
-    server_alpn: Optional[bytes]
+    client_alpn: bytes | None
+    server_alpn: bytes | None
     http2: bool
 
 
@@ -132,6 +143,20 @@ class TlsConfig:
             choices=[x.name for x in net_tls.Version],
             help=f"Set the maximum TLS version for server connections.",
         )
+        loader.add_option(
+            name="tls_ecdh_curve_client",
+            typespec=str | None,
+            default=None,
+            help="Use a specific elliptic curve for ECDHE key exchange on client connections. "
+            'OpenSSL syntax, for example "prime256v1" (see `openssl ecparam -list_curves`).',
+        )
+        loader.add_option(
+            name="tls_ecdh_curve_server",
+            typespec=str | None,
+            default=None,
+            help="Use a specific elliptic curve for ECDHE key exchange on server connections. "
+            'OpenSSL syntax, for example "prime256v1" (see `openssl ecparam -list_curves`).',
+        )
 
     def tls_clienthello(self, tls_clienthello: tls.ClientHelloData):
         conn_context = tls_clienthello.context
@@ -163,10 +188,13 @@ class TlsConfig:
             extra_chain_certs = []
 
         ssl_ctx = net_tls.create_client_proxy_context(
-            method=net_tls.Method.DTLS_SERVER_METHOD if tls_start.is_dtls else net_tls.Method.TLS_SERVER_METHOD,
+            method=net_tls.Method.DTLS_SERVER_METHOD
+            if tls_start.is_dtls
+            else net_tls.Method.TLS_SERVER_METHOD,
             min_version=net_tls.Version[ctx.options.tls_version_client_min],
             max_version=net_tls.Version[ctx.options.tls_version_client_max],
             cipher_list=tuple(cipher_list),
+            ecdh_curve=ctx.options.tls_ecdh_curve_client,
             chain_file=entry.chain_file,
             request_client_cert=False,
             alpn_select_callback=alpn_select_callback,
@@ -176,7 +204,9 @@ class TlsConfig:
         tls_start.ssl_conn = SSL.Connection(ssl_ctx)
 
         tls_start.ssl_conn.use_certificate(entry.cert.to_pyopenssl())
-        tls_start.ssl_conn.use_privatekey(crypto.PKey.from_cryptography_key(entry.privatekey))
+        tls_start.ssl_conn.use_privatekey(
+            crypto.PKey.from_cryptography_key(entry.privatekey)
+        )
 
         # Force HTTP/1 for secure web proxies, we currently don't support CONNECT over HTTP/2.
         # There is a proof-of-concept branch at https://github.com/mhils/mitmproxy/tree/http2-proxy,
@@ -184,7 +214,7 @@ class TlsConfig:
         if len(tls_start.context.layers) == 2 and isinstance(
             tls_start.context.layers[0], modes.HttpProxy
         ):
-            client_alpn: Optional[bytes] = b"http/1.1"
+            client_alpn: bytes | None = b"http/1.1"
         else:
             client_alpn = client.alpn
 
@@ -241,7 +271,7 @@ class TlsConfig:
         # don't assign to client.cipher_list, doesn't need to be stored.
         cipher_list = server.cipher_list or DEFAULT_CIPHERS
 
-        client_cert: Optional[str] = None
+        client_cert: str | None = None
         if ctx.options.client_certs:
             client_certs = os.path.expanduser(ctx.options.client_certs)
             if os.path.isfile(client_certs):
@@ -253,14 +283,18 @@ class TlsConfig:
                     client_cert = p
 
         ssl_ctx = net_tls.create_proxy_server_context(
-            method=net_tls.Method.DTLS_CLIENT_METHOD if tls_start.is_dtls else net_tls.Method.TLS_CLIENT_METHOD,
+            method=net_tls.Method.DTLS_CLIENT_METHOD
+            if tls_start.is_dtls
+            else net_tls.Method.TLS_CLIENT_METHOD,
             min_version=net_tls.Version[ctx.options.tls_version_server_min],
             max_version=net_tls.Version[ctx.options.tls_version_server_max],
             cipher_list=tuple(cipher_list),
+            ecdh_curve=ctx.options.tls_ecdh_curve_server,
             verify=verify,
             ca_path=ctx.options.ssl_verify_upstream_trusted_confdir,
             ca_pemfile=ctx.options.ssl_verify_upstream_trusted_ca,
             client_cert=client_cert,
+            legacy_server_connect=ctx.options.ssl_insecure,
         )
 
         tls_start.ssl_conn = SSL.Connection(ssl_ctx)
@@ -280,7 +314,9 @@ class TlsConfig:
             except ValueError:
                 host_name = server.sni.encode("idna")
                 tls_start.ssl_conn.set_tlsext_host_name(host_name)
-                ok = SSL._lib.X509_VERIFY_PARAM_set1_host(param, host_name, len(host_name))  # type: ignore
+                ok = SSL._lib.X509_VERIFY_PARAM_set1_host(  # type: ignore
+                    param, host_name, len(host_name)
+                )  # type: ignore
                 SSL._openssl_assert(ok == 1)  # type: ignore
             else:
                 # RFC 6066: Literal IPv4 and IPv6 addresses are not permitted in "HostName",
@@ -295,54 +331,157 @@ class TlsConfig:
 
         tls_start.ssl_conn.set_connect_state()
 
+    def quic_start_client(self, tls_start: quic.QuicTlsData) -> None:
+        """Establish QUIC between client and proxy."""
+        if tls_start.settings is not None:
+            return  # a user addon has already provided the settings.
+        tls_start.settings = quic.QuicTlsSettings()
+
+        # keep the following part in sync with `tls_start_client`
+        assert isinstance(tls_start.conn, connection.Client)
+
+        client: connection.Client = tls_start.conn
+        server: connection.Server = tls_start.context.server
+
+        entry = self.get_cert(tls_start.context)
+
+        if not client.cipher_list and ctx.options.ciphers_client:
+            client.cipher_list = ctx.options.ciphers_client.split(":")
+
+        if ctx.options.add_upstream_certs_to_client_chain:  # pragma: no cover
+            extra_chain_certs = server.certificate_list
+        else:
+            extra_chain_certs = []
+
+        # set context parameters
+        if client.cipher_list:
+            tls_start.settings.cipher_suites = [
+                CipherSuite[cipher] for cipher in client.cipher_list
+            ]
+        # if we don't have upstream ALPN, we allow all offered by the client
+        tls_start.settings.alpn_protocols = [
+            alpn.decode("ascii")
+            for alpn in [alpn for alpn in (client.alpn, server.alpn) if alpn]
+            or client.alpn_offers
+        ]
+
+        # set the certificates
+        tls_start.settings.certificate = entry.cert._cert
+        tls_start.settings.certificate_private_key = entry.privatekey
+        tls_start.settings.certificate_chain = [
+            cert._cert for cert in (*entry.chain_certs, *extra_chain_certs)
+        ]
+
+    def quic_start_server(self, tls_start: quic.QuicTlsData) -> None:
+        """Establish QUIC between proxy and server."""
+        if tls_start.settings is not None:
+            return  # a user addon has already provided the settings.
+        tls_start.settings = quic.QuicTlsSettings()
+
+        # keep the following part in sync with `tls_start_server`
+        assert isinstance(tls_start.conn, connection.Server)
+
+        client: connection.Client = tls_start.context.client
+        server: connection.Server = tls_start.conn
+        assert server.address
+
+        if ctx.options.ssl_insecure:
+            tls_start.settings.verify_mode = ssl.CERT_NONE
+        else:
+            tls_start.settings.verify_mode = ssl.CERT_REQUIRED
+
+        if server.sni is None:
+            server.sni = client.sni or server.address[0]
+
+        if not server.alpn_offers:
+            if client.alpn_offers:
+                server.alpn_offers = tuple(client.alpn_offers)
+            else:
+                # aioquic fails if no ALPN is offered, so use H3
+                server.alpn_offers = tuple(alpn.encode("ascii") for alpn in H3_ALPN)
+
+        if not server.cipher_list and ctx.options.ciphers_server:
+            server.cipher_list = ctx.options.ciphers_server.split(":")
+
+        # set context parameters
+        if server.cipher_list:
+            tls_start.settings.cipher_suites = [
+                CipherSuite[cipher] for cipher in server.cipher_list
+            ]
+        if server.alpn_offers:
+            tls_start.settings.alpn_protocols = [
+                alpn.decode("ascii") for alpn in server.alpn_offers
+            ]
+
+        # set the certificates
+        # NOTE client certificates are not supported
+        tls_start.settings.ca_path = ctx.options.ssl_verify_upstream_trusted_confdir
+        tls_start.settings.ca_file = ctx.options.ssl_verify_upstream_trusted_ca
+
     def running(self):
         # FIXME: We have a weird bug where the contract for configure is not followed and it is never called with
         # confdir or command_history as updated.
         self.configure("confdir")  # pragma: no cover
 
     def configure(self, updated):
-        if "confdir" not in updated and "certs" not in updated:
-            return
-
-        certstore_path = os.path.expanduser(ctx.options.confdir)
-        self.certstore = certs.CertStore.from_store(
-            path=certstore_path,
-            basename=CONF_BASENAME,
-            key_size=ctx.options.key_size,
-            passphrase=ctx.options.cert_passphrase.encode("utf8")
-            if ctx.options.cert_passphrase
-            else None,
-        )
-        if self.certstore.default_ca.has_expired():
-            logging.warning(
-                "The mitmproxy certificate authority has expired!\n"
-                "Please delete all CA-related files in your ~/.mitmproxy folder.\n"
-                "The CA will be regenerated automatically after restarting mitmproxy.\n"
-                "See https://docs.mitmproxy.org/stable/concepts-certificates/ for additional help.",
+        if (
+            "certs" in updated
+            or "confdir" in updated
+            or "key_size" in updated
+            or "cert_passphrase" in updated
+        ):
+            certstore_path = os.path.expanduser(ctx.options.confdir)
+            self.certstore = certs.CertStore.from_store(
+                path=certstore_path,
+                basename=CONF_BASENAME,
+                key_size=ctx.options.key_size,
+                passphrase=ctx.options.cert_passphrase.encode("utf8")
+                if ctx.options.cert_passphrase
+                else None,
             )
-
-        for certspec in ctx.options.certs:
-            parts = certspec.split("=", 1)
-            if len(parts) == 1:
-                parts = ["*", parts[0]]
-
-            cert = Path(parts[1]).expanduser()
-            if not cert.exists():
-                raise exceptions.OptionsError(
-                    f"Certificate file does not exist: {cert}"
+            if self.certstore.default_ca.has_expired():
+                logging.warning(
+                    "The mitmproxy certificate authority has expired!\n"
+                    "Please delete all CA-related files in your ~/.mitmproxy folder.\n"
+                    "The CA will be regenerated automatically after restarting mitmproxy.\n"
+                    "See https://docs.mitmproxy.org/stable/concepts-certificates/ for additional help.",
                 )
-            try:
-                self.certstore.add_cert_file(
-                    parts[0],
-                    cert,
-                    passphrase=ctx.options.cert_passphrase.encode("utf8")
-                    if ctx.options.cert_passphrase
-                    else None,
-                )
-            except ValueError as e:
-                raise exceptions.OptionsError(
-                    f"Invalid certificate format for {cert}: {e}"
-                ) from e
+
+            for certspec in ctx.options.certs:
+                parts = certspec.split("=", 1)
+                if len(parts) == 1:
+                    parts = ["*", parts[0]]
+
+                cert = Path(parts[1]).expanduser()
+                if not cert.exists():
+                    raise exceptions.OptionsError(
+                        f"Certificate file does not exist: {cert}"
+                    )
+                try:
+                    self.certstore.add_cert_file(
+                        parts[0],
+                        cert,
+                        passphrase=ctx.options.cert_passphrase.encode("utf8")
+                        if ctx.options.cert_passphrase
+                        else None,
+                    )
+                except ValueError as e:
+                    raise exceptions.OptionsError(
+                        f"Invalid certificate format for {cert}: {e}"
+                    ) from e
+
+        if "tls_ecdh_curve_client" in updated or "tls_ecdh_curve_server" in updated:
+            for ecdh_curve in [
+                ctx.options.tls_ecdh_curve_client,
+                ctx.options.tls_ecdh_curve_server,
+            ]:
+                if ecdh_curve is not None:
+                    try:
+                        crypto.get_elliptic_curve(ecdh_curve)
+                    except Exception as e:
+                        raise exceptions.OptionsError(
+                            f"Invalid ECDH curve: {ecdh_curve!r}"
+                        ) from e
 
     def get_cert(self, conn_context: context.Context) -> certs.CertStoreEntry:
         """
@@ -350,7 +489,7 @@ class TlsConfig:
         our certificate should have and then fetches a matching cert from the certstore.
         """
         altnames: list[str] = []
-        organization: Optional[str] = None
+        organization: str | None = None
 
         # Use upstream certificate if available.
         if ctx.options.upstream_cert and conn_context.server.certificate_list:
@@ -361,17 +500,16 @@ class TlsConfig:
             if upstream_cert.organization:
                 organization = upstream_cert.organization
 
-        # Add SNI. If not available, try the server address as well.
+        # Add SNI or our local IP address.
         if conn_context.client.sni:
             altnames.append(conn_context.client.sni)
-        elif conn_context.server.address:
-            altnames.append(conn_context.server.address[0])
-
-        # As a last resort, add our local IP address. This may be necessary for HTTPS Proxies which are addressed
-        # via IP. Here we neither have an upstream cert, nor can an IP be included in the server name indication.
-        if not altnames:
+        else:
             altnames.append(conn_context.client.sockname[0])
 
+        # If we already know of a server address, include that in the SANs as well.
+        if conn_context.server.address:
+            altnames.append(conn_context.server.address[0])
+
         # only keep first occurrence of each hostname
         altnames = list(dict.fromkeys(altnames))
 
