@@ -3,10 +3,14 @@ from __future__ import annotations
 import asyncio
 import logging
 import socket
-from typing import Any, Callable, Optional, Union, cast
+from collections.abc import Callable
+from typing import Any
+from typing import cast
+from typing import Union
+
+import mitmproxy_rs
 
 from mitmproxy.connection import Address
-from mitmproxy.net import udp_wireguard
 from mitmproxy.utils import human
 
 logger = logging.getLogger(__name__)
@@ -114,7 +118,7 @@ class UdpServer(DrainableDatagramProtocol):
 
 
 class DatagramReader:
-    _packets: asyncio.Queue
+    _packets: asyncio.Queue[bytes]
     _eof: bool
 
     def __init__(self) -> None:
@@ -158,14 +162,14 @@ class DatagramReader:
 
 
 class DatagramWriter:
-    _transport: asyncio.DatagramTransport
+    _transport: asyncio.DatagramTransport | mitmproxy_rs.DatagramTransport
     _remote_addr: Address
     _reader: DatagramReader | None
     _closed: asyncio.Event | None
 
     def __init__(
         self,
-        transport: asyncio.DatagramTransport,
+        transport: asyncio.DatagramTransport | mitmproxy_rs.DatagramTransport,
         remote_addr: Address,
         reader: DatagramReader | None = None,
     ) -> None:
@@ -183,7 +187,9 @@ class DatagramWriter:
             self._closed = None
 
     @property
-    def _protocol(self) -> DrainableDatagramProtocol | udp_wireguard.WireGuardDatagramTransport:
+    def _protocol(
+        self,
+    ) -> DrainableDatagramProtocol | mitmproxy_rs.DatagramTransport:
         return self._transport.get_protocol()  # type: ignore
 
     def write(self, data: bytes) -> None:
@@ -246,10 +252,7 @@ async def start_server(
 ) -> UdpServer:
     """UDP variant of asyncio.start_server."""
 
-    if host == "":
-        # binding to an empty string does not work on Windows or Ubuntu.
-        host = "0.0.0.0"
-
+    assert host, "Cannot bind to an empty host for UDP sockets on Windows or Ubuntu."
     loop = asyncio.get_running_loop()
     _, protocol = await loop.create_datagram_endpoint(
         lambda: UdpServer(datagram_received_cb, loop),
@@ -260,7 +263,7 @@ async def start_server(
 
 
 async def open_connection(
-    host: str, port: int, *, local_addr: Optional[Address] = None
+    host: str, port: int, *, local_addr: Address | None = None
 ) -> tuple[DatagramReader, DatagramWriter]:
     """UDP variant of asyncio.open_connection."""
 
