@@ -1,18 +1,24 @@
 from __future__ import annotations
+
 import contextlib
 import copy
-import weakref
-from collections.abc import Callable, Iterable, Sequence
-from dataclasses import dataclass
 import os
 import pprint
 import textwrap
-from typing import Any, Optional, TextIO, Union
+import weakref
+from collections.abc import Callable
+from collections.abc import Iterable
+from collections.abc import Sequence
+from dataclasses import dataclass
+from typing import Any
+from typing import Optional
+from typing import TextIO
 
 import ruamel.yaml
 
 from mitmproxy import exceptions
-from mitmproxy.utils import signals, typecheck
+from mitmproxy.utils import signals
+from mitmproxy.utils import typecheck
 
 """
     The base implementation for Options.
@@ -27,10 +33,10 @@ class _Option:
     def __init__(
         self,
         name: str,
-        typespec: Union[type, object],  # object for Optional[x], which is not a type.
+        typespec: type | object,  # object for Optional[x], which is not a type.
         default: Any,
         help: str,
-        choices: Optional[Sequence[str]],
+        choices: Sequence[str] | None,
     ) -> None:
         typecheck.check_option_type(name, default, typespec)
         self.name = name
@@ -103,7 +109,7 @@ class OptManager:
     mutation doesn't change the option state inadvertently.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.deferred: dict[str, Any] = {}
         self.changed = signals.SyncSignal(_sig_changed_spec)
         self.changed.connect(self._notify_subscribers)
@@ -116,10 +122,10 @@ class OptManager:
     def add_option(
         self,
         name: str,
-        typespec: Union[type, object],
+        typespec: type | object,
         default: Any,
         help: str,
-        choices: Optional[Sequence[str]] = None,
+        choices: Sequence[str] | None = None,
     ) -> None:
         self._options[name] = _Option(name, typespec, default, help, choices)
         self.changed.send(updated={name})
@@ -150,13 +156,11 @@ class OptManager:
             if i not in self._options:
                 raise exceptions.OptionsError("No such option: %s" % i)
 
-        self._subscriptions.append(
-            (signals.make_weak_ref(func), set(opts))
-        )
+        self._subscriptions.append((signals.make_weak_ref(func), set(opts)))
 
     def _notify_subscribers(self, updated) -> None:
         cleanup = False
-        for (ref, opts) in self._subscriptions:
+        for ref, opts in self._subscriptions:
             callback = ref()
             if callback is not None:
                 if opts & updated:
@@ -368,7 +372,7 @@ class OptManager:
                 f"Received multiple values for {o.name}: {values}"
             )
 
-        optstr: Optional[str]
+        optstr: str | None
         if values:
             optstr = values[0]
         else:
@@ -411,9 +415,9 @@ class OptManager:
 
         o = self._options[optname]
 
-        def mkf(l, s):
-            l = l.replace("_", "-")
-            f = ["--%s" % l]
+        def mkf(x, s):
+            x = x.replace("_", "-")
+            f = ["--%s" % x]
             if s:
                 f.append("-" + s)
             return f
@@ -526,7 +530,7 @@ def parse(text):
             snip = v.problem_mark.get_snippet()
             raise exceptions.OptionsError(
                 "Config error at line %s:\n%s\n%s"
-                % (v.problem_mark.line + 1, snip, v.problem)
+                % (v.problem_mark.line + 1, snip, getattr(v, "problem", ""))
             )
         else:
             raise exceptions.OptionsError("Could not parse options.")
@@ -606,5 +610,5 @@ def save(opts: OptManager, path: str, defaults: bool = False) -> None:
     else:
         data = ""
 
-    with open(path, "wt", encoding="utf8") as f:
+    with open(path, "w", encoding="utf8") as f:
         serialize(opts, f, data, defaults)
