@@ -12,29 +12,46 @@ Example:
 from __future__ import annotations
 
 import asyncio
+import errno
 import json
 import logging
+import os
 import socket
+import sys
 import textwrap
 import typing
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
+from abc import abstractmethod
 from contextlib import contextmanager
 from pathlib import Path
-from typing import ClassVar, Generic, TypeVar, cast, get_args
+from typing import cast
+from typing import ClassVar
+from typing import Generic
+from typing import get_args
+from typing import TypeVar
 
-import errno
-import mitmproxy_wireguard as wg
+import mitmproxy_rs
 
-from mitmproxy import ctx, flow, platform
+from mitmproxy import ctx
+from mitmproxy import flow
+from mitmproxy import platform
 from mitmproxy.connection import Address
 from mitmproxy.master import Master
-from mitmproxy.net import local_ip, udp
-from mitmproxy.net.udp_wireguard import WireGuardDatagramTransport
-from mitmproxy.proxy import commands, layers, mode_specs, server
+from mitmproxy.net import local_ip
+from mitmproxy.net import udp
+from mitmproxy.proxy import commands
+from mitmproxy.proxy import layers
+from mitmproxy.proxy import mode_specs
+from mitmproxy.proxy import server
 from mitmproxy.proxy.context import Context
 from mitmproxy.proxy.layer import Layer
 from mitmproxy.utils import human
 
+if sys.version_info < (3, 11):
+    from typing_extensions import Self  # pragma: no cover
+else:
+    from typing import Self
+
 logger = logging.getLogger(__name__)
 
 
@@ -55,33 +72,33 @@ class ProxyConnectionHandler(server.LiveConnectionHandler):
                 await data.wait_for_resume()  # pragma: no cover
 
 
-M = TypeVar('M', bound=mode_specs.ProxyMode)
+M = TypeVar("M", bound=mode_specs.ProxyMode)
 
 
 class ServerManager(typing.Protocol):
-    connections: dict[tuple, ProxyConnectionHandler]
+    # temporary workaround: for UDP, we use the 4-tuple because we don't have a uuid.
+    connections: dict[tuple | str, ProxyConnectionHandler]
 
     @contextmanager
-    def register_connection(self, connection_id: tuple, handler: ProxyConnectionHandler):
+    def register_connection(
+        self, connection_id: tuple | str, handler: ProxyConnectionHandler
+    ):
         ...  # pragma: no cover
 
 
-# Python 3.11: Use typing.Self
-Self = TypeVar("Self", bound="ServerInstance")
-
-
 class ServerInstance(Generic[M], metaclass=ABCMeta):
     __modes: ClassVar[dict[str, type[ServerInstance]]] = {}
 
+    last_exception: Exception | None = None
+
     def __init__(self, mode: M, manager: ServerManager):
         self.mode: M = mode
         self.manager: ServerManager = manager
-        self.last_exception: Exception | None = None
 
     def __init_subclass__(cls, **kwargs):
         """Register all subclasses so that make() finds them."""
         # extract mode from Generic[Mode].
-        mode = get_args(cls.__orig_bases__[0])[0]
+        mode = get_args(cls.__orig_bases__[0])[0]  # type: ignore
         if not isinstance(mode, TypeVar):
             assert issubclass(mode, mode_specs.ProxyMode)
             assert mode.type_name not in ServerInstance.__modes
@@ -89,7 +106,7 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
 
     @classmethod
     def make(
-        cls: typing.Type[Self],
+        cls,
         mode: mode_specs.ProxyMode | str,
         manager: ServerManager,
     ) -> Self:
@@ -107,12 +124,41 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
     def is_running(self) -> bool:
         pass
 
-    @abstractmethod
     async def start(self) -> None:
+        try:
+            await self._start()
+        except Exception as e:
+            self.last_exception = e
+            raise
+        else:
+            self.last_exception = None
+        if self.listen_addrs:
+            addrs = " and ".join({human.format_address(a) for a in self.listen_addrs})
+            logger.info(f"{self.mode.description} listening at {addrs}.")
+        else:
+            logger.info(f"{self.mode.description} started.")
+
+    async def stop(self) -> None:
+        listen_addrs = self.listen_addrs
+        try:
+            await self._stop()
+        except Exception as e:
+            self.last_exception = e
+            raise
+        else:
+            self.last_exception = None
+        if listen_addrs:
+            addrs = " and ".join({human.format_address(a) for a in listen_addrs})
+            logger.info(f"{self.mode.description} at {addrs} stopped.")
+        else:
+            logger.info(f"{self.mode.description} stopped.")
+
+    @abstractmethod
+    async def _start(self) -> None:
         pass
 
     @abstractmethod
-    async def stop(self) -> None:
+    async def _stop(self) -> None:
         pass
 
     @property
@@ -136,8 +182,8 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
 
     async def handle_tcp_connection(
         self,
-        reader: asyncio.StreamReader | wg.TcpStream,
-        writer: asyncio.StreamWriter | wg.TcpStream,
+        reader: asyncio.StreamReader | mitmproxy_rs.TcpStream,
+        writer: asyncio.StreamWriter | mitmproxy_rs.TcpStream,
     ) -> None:
         handler = ProxyConnectionHandler(
             ctx.master, reader, writer, ctx.options, self.mode
@@ -154,27 +200,23 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
             else:
                 handler.layer.context.client.sockname = original_dst
                 handler.layer.context.server.address = original_dst
-        elif isinstance(self.mode, mode_specs.WireGuardMode):
-            original_dst = writer.get_extra_info("original_dst")
-            handler.layer.context.client.sockname = original_dst
-            handler.layer.context.server.address = original_dst
+        elif isinstance(self.mode, (mode_specs.WireGuardMode, mode_specs.LocalMode)):
+            handler.layer.context.server.address = writer.get_extra_info(
+                "destination_address", handler.layer.context.client.sockname
+            )
 
-        connection_id = (
-            handler.layer.context.client.transport_protocol,
-            handler.layer.context.client.peername,
-            handler.layer.context.client.sockname,
-        )
-        with self.manager.register_connection(connection_id, handler):
+        with self.manager.register_connection(handler.layer.context.client.id, handler):
             await handler.handle_client()
 
     def handle_udp_datagram(
         self,
-        transport: asyncio.DatagramTransport,
+        transport: asyncio.DatagramTransport | mitmproxy_rs.DatagramTransport,
         data: bytes,
         remote_addr: Address,
         local_addr: Address,
     ) -> None:
-        connection_id = ("udp", remote_addr, local_addr)
+        # temporary workaround: we don't have a client uuid here.
+        connection_id = (remote_addr, local_addr)
         if connection_id not in self.manager.connections:
             reader = udp.DatagramReader()
             writer = udp.DatagramWriter(transport, remote_addr, reader)
@@ -185,71 +227,72 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
             handler.layer = self.make_top_layer(handler.layer.context)
             handler.layer.context.client.transport_protocol = "udp"
             handler.layer.context.server.transport_protocol = "udp"
-            if isinstance(self.mode, mode_specs.WireGuardMode):
+            if isinstance(self.mode, (mode_specs.WireGuardMode, mode_specs.LocalMode)):
                 handler.layer.context.server.address = local_addr
 
             # pre-register here - we may get datagrams before the task is executed.
             self.manager.connections[connection_id] = handler
-            asyncio.create_task(self.handle_udp_connection(connection_id, handler))
+            t = asyncio.create_task(self.handle_udp_connection(connection_id, handler))
+            # assign it somewhere so that it does not get garbage-collected.
+            handler._handle_udp_task = t  # type: ignore
         else:
             handler = self.manager.connections[connection_id]
             reader = cast(udp.DatagramReader, handler.transports[handler.client].reader)
         reader.feed_data(data, remote_addr)
 
-    async def handle_udp_connection(self, connection_id: tuple, handler: ProxyConnectionHandler) -> None:
+    async def handle_udp_connection(
+        self, connection_id: tuple, handler: ProxyConnectionHandler
+    ) -> None:
         with self.manager.register_connection(connection_id, handler):
             await handler.handle_client()
 
 
 class AsyncioServerInstance(ServerInstance[M], metaclass=ABCMeta):
-    _server: asyncio.Server | udp.UdpServer | None = None
-    _listen_addrs: tuple[Address, ...] = tuple()
+    _servers: list[asyncio.Server | udp.UdpServer]
+
+    def __init__(self, *args, **kwargs) -> None:
+        self._servers = []
+        super().__init__(*args, **kwargs)
 
     @property
     def is_running(self) -> bool:
-        return self._server is not None
+        return bool(self._servers)
 
-    async def start(self) -> None:
-        assert self._server is None
+    @property
+    def listen_addrs(self) -> tuple[Address, ...]:
+        return tuple(
+            sock.getsockname() for serv in self._servers for sock in serv.sockets
+        )
+
+    async def _start(self) -> None:
+        assert not self._servers
         host = self.mode.listen_host(ctx.options.listen_host)
         port = self.mode.listen_port(ctx.options.listen_port)
         try:
-            self._server = await self.listen(host, port)
-            self._listen_addrs = tuple(s.getsockname() for s in self._server.sockets)
+            self._servers = await self.listen(host, port)
         except OSError as e:
-            self.last_exception = e
             message = f"{self.mode.description} failed to listen on {host or '*'}:{port} with {e}"
             if e.errno == errno.EADDRINUSE and self.mode.custom_listen_port is None:
-                assert self.mode.custom_listen_host is None  # since [@ [listen_addr:]listen_port]
-                message += f"\nTry specifying a different port by using `--mode {self.mode.full_spec}@{port + 1}`."
+                assert (
+                    self.mode.custom_listen_host is None
+                )  # since [@ [listen_addr:]listen_port]
+                message += f"\nTry specifying a different port by using `--mode {self.mode.full_spec}@{port + 2}`."
             raise OSError(e.errno, message, e.filename) from e
-        except Exception as e:
-            self.last_exception = e
-            raise
-        else:
-            self.last_exception = None
-        addrs = " and ".join({human.format_address(a) for a in self._listen_addrs})
-        logger.info(f"{self.mode.description} listening at {addrs}.")
 
-    async def stop(self) -> None:
-        assert self._server is not None
-        # we always reset _server and _listen_addrs and ignore failures
-        server = self._server
-        listen_addrs = self._listen_addrs
-        self._server = None
-        self._listen_addrs = tuple()
+    async def _stop(self) -> None:
+        assert self._servers
         try:
-            server.close()
-            await server.wait_closed()
-        except Exception as e:
-            self.last_exception = e
-            raise
-        else:
-            self.last_exception = None
-        addrs = " and ".join({human.format_address(a) for a in listen_addrs})
-        logger.info(f"Stopped {self.mode.description} at {addrs}.")
+            for s in self._servers:
+                s.close()
+            # https://github.com/python/cpython/issues/104344
+            # await asyncio.gather(*[s.wait_closed() for s in self._servers])
+        finally:
+            # we always reset _server and ignore failures
+            self._servers = []
 
-    async def listen(self, host: str, port: int) -> asyncio.Server | udp.UdpServer:
+    async def listen(
+        self, host: str, port: int
+    ) -> list[asyncio.Server | udp.UdpServer]:
         if self.mode.transport_protocol == "tcp":
             # workaround for https://github.com/python/cpython/issues/89856:
             # We want both IPv4 and IPv6 sockets to bind to the same port.
@@ -261,30 +304,48 @@ class AsyncioServerInstance(ServerInstance[M], metaclass=ABCMeta):
                     s.bind(("", 0))
                     fixed_port = s.getsockname()[1]
                     s.close()
-                    return await asyncio.start_server(self.handle_tcp_connection, host, fixed_port)
+                    return [
+                        await asyncio.start_server(
+                            self.handle_tcp_connection, host, fixed_port
+                        )
+                    ]
                 except Exception as e:
-                    logger.debug(f"Failed to listen on a single port ({e!r}), falling back to default behavior.")
-            return await asyncio.start_server(self.handle_tcp_connection, host, port)
+                    logger.debug(
+                        f"Failed to listen on a single port ({e!r}), falling back to default behavior."
+                    )
+            return [await asyncio.start_server(self.handle_tcp_connection, host, port)]
         elif self.mode.transport_protocol == "udp":
-            # create_datagram_endpoint only creates one socket, so the workaround above doesn't apply
-            # NOTE once we do dual servers, we should consider creating sockets manually to ensure
-            # both TCP and UDP listen to the same IPs and same ports
-            return await udp.start_server(
-                self.handle_udp_datagram,
-                host,
-                port,
-            )
+            # create_datagram_endpoint only creates one (non-dual-stack) socket, so we spawn two servers instead.
+            if not host:
+                ipv4 = await udp.start_server(
+                    self.handle_udp_datagram,
+                    "0.0.0.0",
+                    port,
+                )
+                try:
+                    ipv6 = await udp.start_server(
+                        self.handle_udp_datagram,
+                        "::",
+                        port or ipv4.sockets[0].getsockname()[1],
+                    )
+                except Exception:  # pragma: no cover
+                    logger.debug("Failed to listen on '::', listening on IPv4 only.")
+                    return [ipv4]
+                else:  # pragma: no cover
+                    return [ipv4, ipv6]
+            return [
+                await udp.start_server(
+                    self.handle_udp_datagram,
+                    host,
+                    port,
+                )
+            ]
         else:
             raise AssertionError(self.mode.transport_protocol)
 
-    @property
-    def listen_addrs(self) -> tuple[Address, ...]:
-        return self._listen_addrs
-
 
 class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
-    _server: wg.Server | None = None
-    _listen_addrs: tuple[Address, ...] = tuple()
+    _server: mitmproxy_rs.WireGuardServer | None = None
 
     server_key: str
     client_key: str
@@ -296,7 +357,14 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
     def is_running(self) -> bool:
         return self._server is not None
 
-    async def start(self) -> None:
+    @property
+    def listen_addrs(self) -> tuple[Address, ...]:
+        if self._server:
+            return (self._server.getsockname(),)
+        else:
+            return tuple()
+
+    async def _start(self) -> None:
         assert self._server is None
         host = self.mode.listen_host(ctx.options.listen_host)
         port = self.mode.listen_port(ctx.options.listen_port)
@@ -306,98 +374,143 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
         else:
             conf_path = Path(ctx.options.confdir).expanduser() / "wireguard.conf"
 
-        try:
-            if not conf_path.exists():
-                conf_path.write_text(json.dumps({
-                    "server_key": wg.genkey(),
-                    "client_key": wg.genkey(),
-                }, indent=4))
-
-            try:
-                c = json.loads(conf_path.read_text())
-                self.server_key = c["server_key"]
-                self.client_key = c["client_key"]
-            except Exception as e:
-                raise ValueError(f"Invalid configuration file ({conf_path}): {e}") from e
-            # error early on invalid keys
-            p = wg.pubkey(self.client_key)
-            _ = wg.pubkey(self.server_key)
-
-            self._server = await wg.start_server(
-                host,
-                port,
-                self.server_key,
-                [p],
-                self.wg_handle_tcp_connection,
-                self.wg_handle_udp_datagram,
+        if not conf_path.exists():
+            conf_path.parent.mkdir(parents=True, exist_ok=True)
+            conf_path.write_text(
+                json.dumps(
+                    {
+                        "server_key": mitmproxy_rs.genkey(),
+                        "client_key": mitmproxy_rs.genkey(),
+                    },
+                    indent=4,
+                )
             )
-            self._listen_addrs = (self._server.getsockname(),)
-        except Exception as e:
-            self.last_exception = e
-            message = f"{self.mode.description} failed to listen on {host or '*'}:{port} with {e}"
-            raise OSError(message) from e
-        else:
-            self.last_exception = None
 
-        addrs = " and ".join({human.format_address(a) for a in self.listen_addrs})
+        try:
+            c = json.loads(conf_path.read_text())
+            self.server_key = c["server_key"]
+            self.client_key = c["client_key"]
+        except Exception as e:
+            raise ValueError(f"Invalid configuration file ({conf_path}): {e}") from e
+        # error early on invalid keys
+        p = mitmproxy_rs.pubkey(self.client_key)
+        _ = mitmproxy_rs.pubkey(self.server_key)
+
+        self._server = await mitmproxy_rs.start_wireguard_server(
+            host,
+            port,
+            self.server_key,
+            [p],
+            self.wg_handle_tcp_connection,
+            self.handle_udp_datagram,
+        )
+
         conf = self.client_conf()
         assert conf
-        logger.info(
-            f"{self.mode.description} listening at {addrs}.\n"
-            + "------------------------------------------------------------\n"
-            + conf
-            + "\n------------------------------------------------------------"
-        )
+        logger.info("-" * 60 + "\n" + conf + "\n" + "-" * 60)
 
     def client_conf(self) -> str | None:
         if not self._server:
             return None
         host = local_ip.get_local_ip() or local_ip.get_local_ip6()
         port = self.mode.listen_port(ctx.options.listen_port)
-        return textwrap.dedent(f"""
+        return textwrap.dedent(
+            f"""
             [Interface]
             PrivateKey = {self.client_key}
             Address = 10.0.0.1/32
             DNS = 10.0.0.53
 
             [Peer]
-            PublicKey = {wg.pubkey(self.server_key)}
+            PublicKey = {mitmproxy_rs.pubkey(self.server_key)}
             AllowedIPs = 0.0.0.0/0
             Endpoint = {host}:{port}
-            """).strip()
+            """
+        ).strip()
 
     def to_json(self) -> dict:
-        return {
-            "wireguard_conf": self.client_conf(),
-            **super().to_json()
-        }
+        return {"wireguard_conf": self.client_conf(), **super().to_json()}
 
-    async def stop(self) -> None:
+    async def _stop(self) -> None:
         assert self._server is not None
-        self._server.close()
-        await self._server.wait_closed()
-        self._server = None
-        self.last_exception = None
+        try:
+            self._server.close()
+            await self._server.wait_closed()
+        finally:
+            self._server = None
 
-        addrs = " and ".join({human.format_address(a) for a in self.listen_addrs})
-        logger.info(f"Stopped {self.mode.description} at {addrs}.")
-
-    @property
-    def listen_addrs(self) -> tuple[Address, ...]:
-        return self._listen_addrs
-
-    async def wg_handle_tcp_connection(self, stream: wg.TcpStream) -> None:
+    async def wg_handle_tcp_connection(self, stream: mitmproxy_rs.TcpStream) -> None:
         await self.handle_tcp_connection(stream, stream)
 
-    def wg_handle_udp_datagram(self, data: bytes, remote_addr: Address, local_addr: Address) -> None:
-        assert self._server is not None
-        transport = WireGuardDatagramTransport(self._server, local_addr, remote_addr)
-        self.handle_udp_datagram(
-            transport,
-            data,
-            remote_addr,
-            local_addr
-        )
+
+class LocalRedirectorInstance(ServerInstance[mode_specs.LocalMode]):
+    _server: ClassVar[mitmproxy_rs.LocalRedirector | None] = None
+    """The local redirector daemon. Will be started once and then reused for all future instances."""
+    _instance: ClassVar[LocalRedirectorInstance | None] = None
+    """The current LocalRedirectorInstance. Will be unset again if an instance is stopped."""
+    listen_addrs = ()
+
+    @property
+    def is_running(self) -> bool:
+        return self._instance is not None
+
+    def make_top_layer(self, context: Context) -> Layer:
+        return layers.modes.TransparentProxy(context)
+
+    @classmethod
+    async def redirector_handle_tcp_connection(
+        cls, stream: mitmproxy_rs.TcpStream
+    ) -> None:
+        if cls._instance is not None:
+            await cls._instance.handle_tcp_connection(stream, stream)
+
+    @classmethod
+    def redirector_handle_datagram(
+        cls,
+        transport: mitmproxy_rs.DatagramTransport,
+        data: bytes,
+        remote_addr: Address,
+        local_addr: Address,
+    ) -> None:
+        if cls._instance is not None:
+            cls._instance.handle_udp_datagram(
+                transport=transport,
+                data=data,
+                remote_addr=remote_addr,
+                local_addr=local_addr,
+            )
+
+    async def _start(self) -> None:
+        if self._instance:
+            raise RuntimeError("Cannot spawn more than one local redirector.")
+
+        if self.mode.data.startswith("!"):
+            spec = f"{self.mode.data},{os.getpid()}"
+        elif self.mode.data:
+            spec = self.mode.data
+        else:
+            spec = f"!{os.getpid()}"
+
+        cls = self.__class__
+        cls._instance = self  # assign before awaiting to avoid races
+        if cls._server is None:
+            try:
+                cls._server = await mitmproxy_rs.start_local_redirector(
+                    cls.redirector_handle_tcp_connection,
+                    cls.redirector_handle_datagram,
+                )
+            except Exception:
+                cls._instance = None
+                raise
+
+        cls._server.set_intercept(spec)
+
+    async def _stop(self) -> None:
+        assert self._instance
+        assert self._server
+        self.__class__._instance = None
+        # We're not shutting down the server because we want to avoid additional UAC prompts.
+        self._server.set_intercept("")
 
 
 class RegularInstance(AsyncioServerInstance[mode_specs.RegularMode]):
@@ -428,3 +541,8 @@ class Socks5Instance(AsyncioServerInstance[mode_specs.Socks5Mode]):
 class DnsInstance(AsyncioServerInstance[mode_specs.DnsMode]):
     def make_top_layer(self, context: Context) -> Layer:
         return layers.DNSLayer(context)
+
+
+# class Http3Instance(AsyncioServerInstance[mode_specs.Http3Mode]):
+#     def make_top_layer(self, context: Context) -> Layer:
+#         return layers.modes.HttpProxy(context)
