@@ -1,14 +1,16 @@
 from __future__ import annotations
+
 import asyncio
 import hashlib
 import json
 import logging
 import os.path
 import re
+from collections.abc import Callable
 from collections.abc import Sequence
 from io import BytesIO
 from itertools import islice
-from typing import ClassVar, Optional, Union
+from typing import ClassVar
 
 import tornado.escape
 import tornado.web
@@ -16,7 +18,9 @@ import tornado.websocket
 
 import mitmproxy.flow
 import mitmproxy.tools.web.master
-from mitmproxy import certs, command, contentviews
+from mitmproxy import certs
+from mitmproxy import command
+from mitmproxy import contentviews
 from mitmproxy import flowfilter
 from mitmproxy import http
 from mitmproxy import io
@@ -25,14 +29,16 @@ from mitmproxy import optmanager
 from mitmproxy import version
 from mitmproxy.dns import DNSFlow
 from mitmproxy.http import HTTPFlow
-from mitmproxy.tcp import TCPFlow, TCPMessage
-from mitmproxy.udp import UDPFlow, UDPMessage
+from mitmproxy.tcp import TCPFlow
+from mitmproxy.tcp import TCPMessage
+from mitmproxy.udp import UDPFlow
+from mitmproxy.udp import UDPMessage
 from mitmproxy.utils.emoji import emoji
 from mitmproxy.utils.strutils import always_str
 from mitmproxy.websocket import WebSocketMessage
 
 
-def cert_to_json(certs: Sequence[certs.Cert]) -> Optional[dict]:
+def cert_to_json(certs: Sequence[certs.Cert]) -> dict | None:
     if not certs:
         return None
     cert = certs[0]
@@ -103,8 +109,8 @@ def flow_to_json(flow: mitmproxy.flow.Flow) -> dict:
         f["error"] = flow.error.get_state()
 
     if isinstance(flow, HTTPFlow):
-        content_length: Optional[int]
-        content_hash: Optional[str]
+        content_length: int | None
+        content_hash: str | None
 
         if flow.request.raw_content is not None:
             content_length = len(flow.request.raw_content)
@@ -191,9 +197,9 @@ class APIError(tornado.web.HTTPError):
 
 
 class RequestHandler(tornado.web.RequestHandler):
-    application: "Application"
+    application: Application
 
-    def write(self, chunk: Union[str, bytes, dict, list]):
+    def write(self, chunk: str | bytes | dict | list):
         # Writing arrays on the top level is ok nowadays.
         # http://flask.pocoo.org/docs/0.11/security/#json-security
         if isinstance(chunk, list):
@@ -237,11 +243,11 @@ class RequestHandler(tornado.web.RequestHandler):
             return self.request.body
 
     @property
-    def view(self) -> "mitmproxy.addons.view.View":
+    def view(self) -> mitmproxy.addons.view.View:
         return self.application.master.view
 
     @property
-    def master(self) -> "mitmproxy.tools.web.master.WebMaster":
+    def master(self) -> mitmproxy.tools.web.master.WebMaster:
         return self.application.master
 
     @property
@@ -276,6 +282,7 @@ class FilterHelp(RequestHandler):
 class WebSocketEventBroadcaster(tornado.websocket.WebSocketHandler):
     # raise an error if inherited class doesn't specify its own instance.
     connections: ClassVar[set[WebSocketEventBroadcaster]]
+    _send_tasks: ClassVar[set[asyncio.Task]] = set()
 
     def open(self):
         self.connections.add(self)
@@ -291,7 +298,9 @@ class WebSocketEventBroadcaster(tornado.websocket.WebSocketHandler):
             except tornado.websocket.WebSocketClosedError:
                 cls.connections.discard(conn)
 
-        asyncio.ensure_future(wrapper())
+        t = asyncio.create_task(wrapper())
+        cls._send_tasks.add(t)
+        t.add_done_callback(cls._send_tasks.remove)
 
     @classmethod
     def broadcast(cls, **kwargs):
@@ -313,16 +322,22 @@ class Flows(RequestHandler):
 
 
 class DumpFlows(RequestHandler):
-    def get(self):
+    def get(self) -> None:
         self.set_header("Content-Disposition", "attachment; filename=flows")
         self.set_header("Content-Type", "application/octet-stream")
 
+        match: Callable[[mitmproxy.flow.Flow], bool]
         try:
             match = flowfilter.parse(self.request.arguments["filter"][0].decode())
         except ValueError:  # thrown py flowfilter.parse if filter is invalid
             raise APIError(400, f"Invalid filter argument / regex")
-        except (KeyError, IndexError):  # Key+Index: ["filter"][0] can fail, if it's not set
-            match = bool  # returns always true
+        except (
+            KeyError,
+            IndexError,
+        ):  # Key+Index: ["filter"][0] can fail, if it's not set
+
+            def match(_) -> bool:
+                return True
 
         with BytesIO() as bio:
             fw = io.FlowWriter(bio)
@@ -331,11 +346,11 @@ class DumpFlows(RequestHandler):
                     fw.add(f)
             self.write(bio.getvalue())
 
-    def post(self):
+    async def post(self):
         self.view.clear()
         bio = BytesIO(self.filecontents)
-        for i in io.FlowReader(bio).stream():
-            asyncio.ensure_future(self.master.load_flow(i))
+        for f in io.FlowReader(bio).stream():
+            await self.master.load_flow(f)
         bio.close()
 
 
@@ -381,7 +396,7 @@ class FlowHandler(RequestHandler):
             self.flow.kill()
         self.view.remove([self.flow])
 
-    def put(self, flow_id):
+    def put(self, flow_id) -> None:
         flow: mitmproxy.flow.Flow = self.flow
         flow.backup()
         try:
@@ -469,13 +484,13 @@ class FlowContent(RequestHandler):
 
     def get(self, flow_id, message):
         message = getattr(self.flow, message)
+        assert isinstance(self.flow, HTTPFlow)
 
         original_cd = message.headers.get("Content-Disposition", None)
         filename = None
         if original_cd:
-            filename = re.search(r'filename=([-\w" .()]+)', original_cd)
-            if filename:
-                filename = filename.group(1)
+            if m := re.search(r'filename=([-\w" .()]+)', original_cd):
+                filename = m.group(1)
         if not filename:
             filename = self.flow.request.path.split("?")[0].split("/")[-1]
 
@@ -492,9 +507,9 @@ class FlowContentView(RequestHandler):
     def message_to_json(
         self,
         viewname: str,
-        message: Union[http.Message, TCPMessage, UDPMessage, WebSocketMessage],
-        flow: Union[HTTPFlow, TCPFlow, UDPFlow],
-        max_lines: Optional[int] = None,
+        message: http.Message | TCPMessage | UDPMessage | WebSocketMessage,
+        flow: HTTPFlow | TCPFlow | UDPFlow,
+        max_lines: int | None = None,
     ):
         description, lines, error = contentviews.get_message_content_view(
             viewname, message, flow
@@ -509,7 +524,7 @@ class FlowContentView(RequestHandler):
             description=description,
         )
 
-    def get(self, flow_id, message, content_view):
+    def get(self, flow_id, message, content_view) -> None:
         flow = self.flow
         assert isinstance(flow, (HTTPFlow, TCPFlow, UDPFlow))
 
@@ -519,6 +534,7 @@ class FlowContentView(RequestHandler):
             max_lines = None
 
         if message == "messages":
+            messages: list[TCPMessage] | list[UDPMessage] | list[WebSocketMessage]
             if isinstance(flow, HTTPFlow) and flow.websocket:
                 messages = flow.websocket.messages
             elif isinstance(flow, (TCPFlow, UDPFlow)):
@@ -544,7 +560,7 @@ class FlowContentView(RequestHandler):
 class Commands(RequestHandler):
     def get(self) -> None:
         commands = {}
-        for (name, cmd) in self.master.commands.commands.items():
+        for name, cmd in self.master.commands.commands.items():
             commands[name] = {
                 "help": cmd.help,
                 "parameters": [
@@ -614,31 +630,35 @@ class DnsRebind(RequestHandler):
         raise tornado.web.HTTPError(
             403,
             reason="To protect against DNS rebinding, mitmweb can only be accessed by IP at the moment. "
-                   "(https://github.com/mitmproxy/mitmproxy/issues/3234)",
+            "(https://github.com/mitmproxy/mitmproxy/issues/3234)",
         )
 
 
 class State(RequestHandler):
     def get(self):
-        self.write({
-            "version": version.VERSION,
-            "contentViews": [v.name for v in contentviews.views if v.name != "Query"],
-            "servers": [s.to_json() for s in self.master.proxyserver.servers]
-        })
+        self.write(
+            {
+                "version": version.VERSION,
+                "contentViews": [
+                    v.name for v in contentviews.views if v.name != "Query"
+                ],
+                "servers": [s.to_json() for s in self.master.proxyserver.servers],
+            }
+        )
 
 
 class GZipContentAndFlowFiles(tornado.web.GZipContentEncoding):
     CONTENT_TYPES = {
         "application/octet-stream",
-        *tornado.web.GZipContentEncoding.CONTENT_TYPES
+        *tornado.web.GZipContentEncoding.CONTENT_TYPES,
     }
 
 
 class Application(tornado.web.Application):
-    master: "mitmproxy.tools.web.master.WebMaster"
+    master: mitmproxy.tools.web.master.WebMaster
 
     def __init__(
-        self, master: "mitmproxy.tools.web.master.WebMaster", debug: bool
+        self, master: mitmproxy.tools.web.master.WebMaster, debug: bool
     ) -> None:
         self.master = master
         super().__init__(
