@@ -1,6 +1,6 @@
+import errno
 import logging
 
-import errno
 import tornado.httpserver
 import tornado.ioloop
 
@@ -8,22 +8,25 @@ from mitmproxy import addons
 from mitmproxy import flow
 from mitmproxy import log
 from mitmproxy import master
+from mitmproxy import options
 from mitmproxy import optmanager
-from mitmproxy.addons import errorcheck, eventstore
+from mitmproxy.addons import errorcheck
+from mitmproxy.addons import eventstore
 from mitmproxy.addons import intercept
 from mitmproxy.addons import readfile
 from mitmproxy.addons import termlog
 from mitmproxy.addons import view
 from mitmproxy.addons.proxyserver import Proxyserver
-from mitmproxy.contrib.tornado import patch_tornado
-from mitmproxy.tools.web import app, webaddons, static_viewer
+from mitmproxy.tools.web import app
+from mitmproxy.tools.web import static_viewer
+from mitmproxy.tools.web import webaddons
 
 logger = logging.getLogger(__name__)
 
 
 class WebMaster(master.Master):
-    def __init__(self, options, with_termlog=True):
-        super().__init__(options)
+    def __init__(self, opts: options.Options, with_termlog: bool = True):
+        super().__init__(opts)
         self.view = view.View()
         self.view.sig_view_add.connect(self._sig_view_add)
         self.view.sig_view_remove.connect(self._sig_view_remove)
@@ -86,22 +89,23 @@ class WebMaster(master.Master):
         app.ClientConnection.broadcast(
             resource="state",
             cmd="update",
-            data={"servers": [s.to_json() for s in self.proxyserver.servers]}
+            data={"servers": [s.to_json() for s in self.proxyserver.servers]},
         )
 
     async def running(self):
-        patch_tornado()
         # Register tornado with the current event loop
         tornado.ioloop.IOLoop.current()
 
         # Add our web app.
-        http_server = tornado.httpserver.HTTPServer(self.app)
+        http_server = tornado.httpserver.HTTPServer(
+            self.app, max_buffer_size=2**32
+        )  # 4GB
         try:
             http_server.listen(self.options.web_port, self.options.web_host)
         except OSError as e:
             message = f"Web server failed to listen on {self.options.web_host or '*'}:{self.options.web_port} with {e}"
             if e.errno == errno.EADDRINUSE:
-                message += f"\nTry specifying a different port by using `--set web_port={self.options.web_port + 1}`."
+                message += f"\nTry specifying a different port by using `--set web_port={self.options.web_port + 2}`."
             raise OSError(e.errno, message, e.filename) from e
 
         logger.info(
